#include "gen.h"

#include "lib.h"
#include <string>
#include <vector>
#include <fstream>
#include <utility>
#include <exception>
using namespace std;

static void newUnexpected () {
	fatErrMsg ( "Unexpected exception." );
}

static void newTerminate () {
	fatErrMsg ( "Uncaught exception." );
}

void initGen () {
	set_unexpected ( newUnexpected );
	set_terminate ( newTerminate );
}
	

string numToText ( int num ) {
	const int RADIX = 10;
	char charBuf [ 20 ];

	#ifdef BCC
		itoa ( num, charBuf, RADIX );
		#define GOODNUMSTR
	#endif
	#ifdef MING
		itoa ( num, charBuf, RADIX );
		#define GOODNUMSTR
	#endif

	#ifndef GOODNUMSTR
		blaaargh!
	#endif
	return string ( charBuf );
}

int textToNum ( string text ) {
	return atoi ( text . c_str () );
}

pair <string, string> stripArgs ( string & line, char leftArg, char rightArg ) throw ( NoArg ) {
	string::size_type leftArgInd = line . find ( leftArg );
	string::size_type rightArgInd = line . find ( rightArg );

	if ( leftArgInd == string::npos || rightArgInd == string::npos )
		throw NoArg ();

	pair <string, string> retPair;
	retPair . first = line . substr ( 0, leftArgInd );

	if ( rightArgInd - leftArgInd == 1 )
		retPair . second . clear ();
	else
		retPair . second = line . substr ( leftArgInd + 1, rightArgInd - leftArgInd - 1 );

	line = line . substr ( rightArgInd + 1, string::npos );
	return retPair;
}

FileIn::FileIn ( string file ) throw ( FileIn::FileError ) :
streamIn ( file . c_str () ) {
	if ( ! streamIn . is_open () )
		throw CouldNotOpen ();

	eofFlag = false;
	getLine ();

	if ( eofFlag ) {
		streamIn . close ();
		throw EmptyFile ();
	}
}

FileIn::~FileIn () {
	streamIn . close ();
}

string FileIn::getLine () throw ( FileIn::ReadPastEof ) {
	if ( eofFlag )
		throw ReadPastEof ();

	string returnLine = lineBuff;
	bool newBuff = false;

	while ( ! streamIn . eof () ) {
		getline ( streamIn, lineBuff );

		if ( ! ( lineBuff . empty () ) && lineBuff [ 0 ] != COMMENT ) {
			newBuff = true;
			break;
		}
	}

	if ( ! newBuff )
		eofFlag = true;

	return returnLine;
}
