#include "gui.h"

#include "lib.h"
#include <string>
using namespace std;

const string GUIBMPDIR ( "gui/" );
const string BKGDBMPFILE ( GUIBMPDIR + "bkgd.bmp" );
const string NBORDBMPFILE ( GUIBMPDIR + "bordern.bmp" );
const string NEBORDBMPFILE ( GUIBMPDIR + "borderne.bmp" );
const string EBORDBMPFILE ( GUIBMPDIR + "bordere.bmp" );
const string SEBORDBMPFILE ( GUIBMPDIR + "borderse.bmp" );
const string SBORDBMPFILE ( GUIBMPDIR + "borders.bmp" );
const string SWBORDBMPFILE ( GUIBMPDIR + "bordersw.bmp" );
const string WBORDBMPFILE ( GUIBMPDIR + "borderw.bmp" );
const string NWBORDBMPFILE ( GUIBMPDIR + "bordernw.bmp" );

Bmp * bkgdBmp = NULL;
Bmp * nBordBmp = NULL;
Bmp * neBordBmp = NULL;
Bmp * eBordBmp = NULL;
Bmp * seBordBmp = NULL;
Bmp * sBordBmp = NULL;
Bmp * swBordBmp = NULL;
Bmp * wBordBmp = NULL;
Bmp * nwBordBmp = NULL;

const int TEXTOFF = 5;
int textOffX1 = 0;
int textOffY1 = 0;
int textOffX2 = 0;
int textOffY2 = 0;

Lib_Clr curTextClr;
Lib_Clr curSelTextClr;

bool guiLoaded = false;

void initGui () {
	try {
		bkgdBmp = new Bmp ( BKGDBMPFILE );
		nBordBmp = new Bmp ( NBORDBMPFILE );
		neBordBmp = new Bmp ( NEBORDBMPFILE );
		eBordBmp = new Bmp ( EBORDBMPFILE );
		seBordBmp = new Bmp ( SEBORDBMPFILE );
		sBordBmp = new Bmp ( SBORDBMPFILE );
		swBordBmp = new Bmp ( SWBORDBMPFILE );
		wBordBmp = new Bmp ( WBORDBMPFILE );
		nwBordBmp = new Bmp ( NWBORDBMPFILE );
	} catch ( Bmp::LoadError err ) {
		fatErrMsg ( "Error loading gui bitmaps.\nCause: " + err . what () );
	}

	textOffX1 = wBordBmp -> retW () + TEXTOFF;
	textOffY1 = nBordBmp -> retH () + TEXTOFF;
	textOffX2 = eBordBmp -> retW () + TEXTOFF;
	textOffY2 = sBordBmp -> retH () + TEXTOFF;

	curTextClr = createClr ( 255, 255, 255 );
	curSelTextClr = createClr ( 255, 255, 0 );

	guiLoaded = true;
}



// GuiBox

GuiBox::GuiBox ( int x1, int y1, int x2, int y2 )
: boxBmp ( x2 - x1, y2 - y1 ) {
	GuiBox::x1 = x1;
	GuiBox::y1 = y1;
	GuiBox::x2 = x2;
	GuiBox::y2 = y2;

	const int BMPX1 = 0;
	const int BMPY1 = 0;
	const int BMPX2 = BMPX1 + x2 - x1;
	const int BMPY2 = BMPY1 + y2 - y1;

	bkgdBmp -> tile ( BMPX1, BMPY1, BMPX2, BMPY2, boxBmp );

	const int NWBORDX1 = BMPX1;
	const int NWBORDY1 = BMPY1;
	const int NWBORDX2 = NWBORDX1 + nwBordBmp -> retW () - 1;
	const int NWBORDY2 = NWBORDY1 + nwBordBmp -> retH () - 1;
	nwBordBmp -> draw ( NWBORDX1, NWBORDY1, boxBmp );

	const int NEBORDX1 = BMPX2 - neBordBmp -> retW ();
	const int NEBORDY1 = BMPY1;
	const int NEBORDX2 = NEBORDX1 + neBordBmp -> retW () - 1;
	const int NEBORDY2 = NEBORDY1 + neBordBmp -> retH () - 1;
	neBordBmp -> draw ( NEBORDX1, NEBORDY1, boxBmp );

	const int SWBORDX1 = BMPX1;
	const int SWBORDY1 = BMPY2 - swBordBmp -> retH ();
	const int SWBORDX2 = SWBORDX1 + swBordBmp -> retW () - 1;
	const int SWBORDY2 = SWBORDY1 + swBordBmp -> retH () - 1;
	swBordBmp -> draw ( SWBORDX1, SWBORDY1, boxBmp );

	const int SEBORDX1 = BMPX2 - seBordBmp -> retW ();
	const int SEBORDY1 = BMPY2 - seBordBmp -> retH ();
	const int SEBORDX2 = SEBORDX1 + seBordBmp -> retW () - 1;
	const int SEBORDY2 = SEBORDY1 + seBordBmp -> retH () - 1;
	seBordBmp -> draw ( SEBORDX1, SEBORDY1, boxBmp );

	const int NBORDX1 = NWBORDX2 + 1;
	const int NBORDY1 = BMPY1;
	const int NBORDX2 = NEBORDX1 - 1;
	const int NBORDY2 = nBordBmp -> retH () - 1;
	nBordBmp -> tile ( NBORDX1, NBORDY1, NBORDX2, NBORDY2, boxBmp );

	const int WBORDX1 = BMPX1;
	const int WBORDY1 = NWBORDY2 + 1;
	const int WBORDX2 = wBordBmp -> retW () - 1;
	const int WBORDY2 = SWBORDY1 - 1;
	wBordBmp -> tile ( WBORDX1, WBORDY1, WBORDX2, WBORDY2, boxBmp );

	const int SBORDX1 = SWBORDX2 + 1;
	const int SBORDY1 = BMPY2 - sBordBmp -> retH ();
	const int SBORDX2 = SEBORDX1 - 1;
	const int SBORDY2 = BMPY2 - 1;
	sBordBmp -> tile ( SBORDX1, SBORDY1, SBORDX2, SBORDY2, boxBmp );

	const int EBORDX1 = BMPX2 - eBordBmp -> retW ();
	const int EBORDY1 = NEBORDY2 + 1;
	const int EBORDX2 = BMPX2 - 1;
	const int EBORDY2 = SEBORDY1 - 1;
	eBordBmp -> tile ( EBORDX1, EBORDY1, EBORDX2, EBORDY2, boxBmp );
}



// VoxBox

VoxBox::VoxBox ( string text, int x, int y )
: GuiBox ( x, y, x + detW ( text ) - 1, y + detH ( text ) - 1 ) {
	vox = text;
}

int VoxBox::detW ( string text ) {
	return ( textOffX1 + textOffX2 + text_length ( font, text . c_str () ) );
}

int VoxBox::detH ( string text ) {
	return ( textOffY1 + textOffY2 + text_height ( font ) );
}

void VoxBox::draw () {
	GuiBox::draw ();
	drawText ( vox, retX1 () + textOffX1, retY1 () + textOffY1 );
}



// MsgBox

const int MsgBox::MSGX = 50;
const int MsgBox::MSGY = 50;
const Lib_Key MsgBox::MSGKEY = KEY_SPACE;

MsgBox::MsgBox ( string text )
: VoxBox ( text, MSGX, MSGY ) {
	VoxBox::draw ();
	screenBmp -> drawToScreen ( 0, 0 );
	keyboard -> waitForTap ( MSGKEY );
}
