#ifndef HEAD_LIB
#define HEAD_LIB

#ifdef BCC
	#include <errno.h>
	#include <limits.h>
	#include <stdarg.h>
	#include <stddef.h>
	#include <stdlib.h>
	#include <time.h>
	#include <allegro.h>
#endif
#ifdef MING
	#include <allegro.h>
#endif

#include "gen.h"
#include <string>

//#define NOMOUSE

void initLib ();



// Error

void errMsg ( std::string msg );
void fatErrMsg ( std::string msg );



// Timer

bool logicDone ();
void displayFPS ();




// Keyboard

typedef int Lib_Key;

class Keyboard {

	private:

	static bool loaded;

	public:

	Keyboard ();

	inline bool press () { return keypressed (); }
	bool press ( Lib_Key keyId ) { return key [ keyId ]; }
	bool tap ( Lib_Key keyId );
	Dir dir ();

	void waitForTap ( Lib_Key keyId );

	std::string textIn ( int maxChars );
	int numIn ( int maxDigits );
} extern * keyboard;




// Mouse

#ifndef NOMOUSE
	class Mouse {

		private:

		static const int MINBTNS = 2;

		static bool loaded;
		bool needPoll;
		inline void poll () { if ( needPoll ) poll_mouse (); }

		public:

		class MouseError {};
			class TooFewBtns : public MouseError {};

		Mouse () throw ( TooFewBtns );

		inline int x () { poll (); return mouse_x; }
		inline int y () { poll (); return mouse_y; }
		inline bool btn ( int btnNum ) { poll (); return (mouse_b & btnNum); }

		inline bool left () { return btn ( 1 ); }
		inline bool right () { return btn ( 2 ); }
		inline bool third () { return btn ( 3 ); }
		bool click ();

		bool inField ( int x1, int y1, int x2, int y2 );
		inline bool clickInField ( int x1, int y1, int x2, int y2 ) { return (inField ( x1, y1, x2, y2 ) && click ()); }

		inline void hide () { scare_mouse (); };
		inline void show () { unscare_mouse (); };

		inline void positionMouse ( int x, int y ) { position_mouse ( x, y ); }
	} extern * mouse;
#endif



// Graphics

typedef int Lib_Clr;
typedef PALETTE Lib_Pal;

extern Lib_Clr maskClr;
extern Lib_Pal pal;

inline Lib_Clr createClr ( int r, int g, int b ) { return makecol ( r, g, b ); }

class Bmp;
extern Bmp * screenBmp;

enum GfxMode { fullscreen, windowed };

class ResError : public Exception { public: ResError ( std::string msg ) : Exception ( msg ) {} };
	class BadRes : public ResError { public: BadRes ( std::string msg ) : ResError ( msg ) {} };
	class BuffError : public ResError { public: BuffError ( std::string msg ) : ResError ( msg ) {} };
	class BadSwitchMode : public ResError { public: BadSwitchMode ( std::string msg ) : ResError ( msg ) {} };

void drawScreen ();
void setScreenRes ( GfxMode gfxMode, int w, int h, int clrDepth = desktop_color_depth () ) throw ( ResError );




// Bmp

typedef BITMAP * Lib_Bmp;

void drawText ( std::string, int x, int y );
class Bmp;
extern Bmp * screenBmp;
class Bmp {

	friend void drawText ( std::string, int x, int y );
	friend void drawScreen ();

	private:

	Bmp ( Bmp & src ) { } // No copy constructor: too, too slow.

	Lib_Bmp bmp;

	public:

	class LoadError : public Exception {
		public: LoadError ( std::string msg ) : Exception ( msg ) {}
	};

	Bmp ( int w, int h, bool video = false ) throw ( LoadError );
	Bmp ( std::string bmpFile, bool video = false ) throw ( LoadError );
	Bmp ( Lib_Bmp, bool video = false ) throw ( LoadError );

	~Bmp ();

	inline int retW () { return bmp -> w; }
	inline int retH () { return bmp -> h; }

	inline void clearToClr ( Lib_Clr clr ) { clear_to_color ( bmp, clr ); }
	inline Lib_Clr clrAtPxl ( int x, int y ) { return getpixel ( bmp, x, y ); }

	inline void lock () { acquire_bitmap ( bmp ); }
	inline void unlock () { release_bitmap ( bmp ); }

	inline void drawToScreen ( int x, int y ) { blit ( bmp, screen, 0, 0, x, y, bmp -> w, bmp -> h ); }

	inline void draw ( int x, int y, Bmp & targBmp = (*screenBmp) ) { blit ( bmp, targBmp . bmp, 0, 0, x, y, bmp -> w, bmp -> h ); }
	inline void drawDeform ( int targX1, int targY1, int targX2, int targY2, Bmp & targBmp = (*screenBmp) ) { stretch_blit ( bmp, targBmp . bmp, 0, 0, bmp -> w, bmp -> h, targX1, targY1, (targX2 - targX1 + 1), (targY2 - targY1 + 1) ); }
	inline void drawPart ( int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2, Bmp & targBmp = (*screenBmp) ) { blit ( bmp, targBmp . bmp, srcX1, srcY1, targX, targY, (srcX2 - srcX1 + 1), (srcY2 - srcY1 + 1) ); }

	inline void drawMask ( int x, int y, Bmp & targBmp = (*screenBmp) ) { masked_blit ( bmp, targBmp . bmp, 0, 0, x, y, bmp -> w, bmp -> h ); }
	inline void drawMaskDeform ( int targX1, int targY1, int targX2, int targY2, Bmp & targBmp = (*screenBmp) ) { masked_stretch_blit ( bmp, targBmp . bmp, 0, 0, bmp -> w, bmp -> h, targX1, targY1, (targX2 - targX1 + 1), (targY2 - targY1 + 1) ); }
	inline void drawMaskPart ( int targX, int targY, int srcX1, int srcY1, int srcX2, int srcY2, Bmp & targBmp = (*screenBmp) ) { masked_blit ( bmp, targBmp . bmp, srcX1, srcY1, targX, targY, (srcX2 - srcX1 + 1), (srcY2 - srcY1 + 1) ); }

	void tile ( int targX1, int targY1, int targX2, int targY2, Bmp & targBmp = (*screenBmp) );
	void tileMask ( int targX1, int targY1, int targX2, int targY2, Bmp & targBmp = (*screenBmp) );

	bool coll ( int selfX, int selfY, int targX, int targY, Bmp & targBmp );
};

#endif
